package com.example.azuretranslation;

import androidx.appcompat.app.AppCompatActivity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Adapter;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.TextView;

import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.google.gson.reflect.TypeToken;

import java.util.ArrayList;
import java.util.Date;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;
import retrofit2.converter.scalars.ScalarsConverterFactory;

public class MainActivity extends AppCompatActivity {

    final static long MILLIS_DAY = 86000000;
    final String TAG = "mytag";
    ArrayList<Language> langArr = new ArrayList<Language>();;
    ArrayAdapter<Language> adapter;
    Gson gson = new Gson();
    DBHelper dbHelper;

    TextView toText, fromText;
    Button transButton, historyButton;
    Spinner spinner;
    // Экземпляр библиотеки и интерфейса можно использовать для всех обращений к сервису
    // формируем экземпляр библиотеки
    Retrofit retrofit = new Retrofit.Builder()
            .addConverterFactory(ScalarsConverterFactory.create())
            .addConverterFactory(GsonConverterFactory.create()) // ответ от сервера в виде строки
            .baseUrl(AzureTranslationAPI.API_URL) // адрес API сервера
            .build();

    AzureTranslationAPI api = retrofit.create(AzureTranslationAPI.class); // описываем, какие функции реализованы

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        toText = findViewById(R.id.toText);
        fromText = findViewById(R.id.fromText);
        transButton = findViewById(R.id.transButton);
        historyButton = findViewById(R.id.historyButton);
        spinner = findViewById(R.id.spinner);
        adapter = new ArrayAdapter<Language>(getApplicationContext(), R.layout.support_simple_spinner_dropdown_item, langArr);
        spinner.setAdapter(adapter);
        dbHelper = new DBHelper(this);

        init();

        transButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String textToTrans = fromText.getText().toString();
                JsonObject obj = new JsonObject();
                obj.addProperty("Text", textToTrans);
                JsonArray arr = new JsonArray();
                arr.add(obj);
                Call<TranslatedTextArray[]> call = api.translate(arr.toString(), spinner.getSelectedItem().toString().split(" ")[1]); // создаём объект-вызов
                call.enqueue(new TranslatedTextCallBack());

            }
        });
        historyButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent i = new Intent(MainActivity.this, ListActivity.class);
                startActivity(i);
            }
        });
    }

    // TODO: создать аналогичным образом класс для ответа сервера при переводе текста
    class LanguagesCallback implements Callback<LanguagesResponse> {

        @Override
        public void onResponse(Call<LanguagesResponse> call, Response<LanguagesResponse> response) {
            Log.d("mytag", "onResponse: " + response.headers());
            if (response.isSuccessful()) {
                // TODO: response.body() содержит массив языков, доступных для перевода
                Log.d("mytag", "response: " + response.body());
                langArr = new ArrayList<>();
                for (String s : response.body().translation.keySet()) {
                    langArr.add(response.body().translation.get(s));
                }
                adapter.notifyDataSetChanged();
                String j = gson.toJson(langArr);
                saveValue("langs", j);
            } else {
                Log.d("mytag", "onResponse: " + "все плохо");
                // TODO: выводить Toast и сообщение в журнал в случае ошибки
            }
        }

        @Override
        public void onFailure(Call<LanguagesResponse> call, Throwable t) {
            Log.d("mytag", "onResponse: " + "все плохо");
        }


    }

    class TranslatedTextCallBack implements Callback<TranslatedTextArray[]> {

        @Override
        public void onResponse(Call<TranslatedTextArray[]> call, Response<TranslatedTextArray[]> response) {
            if (response.isSuccessful()) {
                Log.d("mytag", "response: " + response.body());
                TranslatedTextArray[] arrs = response.body();
                String out = arrs[0].toString();
                toText.setText(out);
                dbHelper.add(fromText.getText().toString(), toText.getText().toString());
            } else {
                // TODO: выводить Toast и сообщение в журнал в случае ошибки
            }
        }

        @Override
        public void onFailure(Call<TranslatedTextArray[]> call, Throwable t) {
            // TODO: выводить Toast и сообщение в журнал в случае ошибки
        }
    }

    void saveValue(String key, String value)
    {
        SharedPreferences sharedPreferences = getSharedPreferences("app_values", MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.putString(key, value);
        editor.commit();
    }

    String loadValue(String key)
    {
        SharedPreferences sharedPreferences = getSharedPreferences("app_values", MODE_PRIVATE);
        return  sharedPreferences.getString(key, "");
    }

     void init()
    {
        SharedPreferences preferences = getSharedPreferences("app_values", MODE_PRIVATE);
        if (!preferences.contains("date"))
            saveValue("date", Long.toString(new Date().getTime()));
        if (!preferences.contains("langs"))
            saveValue("langs", "");
        if (new Date().getTime() - Long.parseLong(loadValue("date")) > MILLIS_DAY || loadValue("langs").equals(""))
        {
            Call<LanguagesResponse> call = api.getLanguages(); // создаём объект-вызов
            call.enqueue(new LanguagesCallback());
            saveValue("date", Long.toString(new Date().getTime()));
        }
        else if (!(new Date().getTime() - Long.parseLong(loadValue("date")) > MILLIS_DAY))
        {
            String jsonStr = loadValue("langs");
            langArr.addAll(gson.fromJson(jsonStr, new TypeToken<ArrayList<Language>>(){}.getType()));
            adapter.notifyDataSetChanged();
        }

    }

    void saveToDB() {}
    void loadFromDB(){}
}
